----------------------------------------------------------------------------------
-- Company: Yale University & Artemis Sci.
-- Engineer: Ge Yang
-- 
-- Create Date:    21:40:18 09/03/2010 
-- Design Name:     Arbitrary Waveform Generator
-- Module Name:    Signal_Generator - Behavioral 
-- Project Name:     Independent AWG
-- Target Devices: Spartan3AN700
-- Tool versions:  Xilinx ISE 12.1
-- Description:        
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments:
-- 20100911 Added Square Modulation out through DAC_A
-- 20100908 This version is BUG-FREE. 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL; --This is fine, because this module do all calculation without needed signed multiplication.
use IEEE.numeric_std.all; 
use IEEE.STD_LOGIC_ARITH.ALL;
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
library unisim;
use unisim.vcomponents.all;

entity Signal_Generator is
    Port ( AWG_CLK,Square_wave_handle: in  STD_LOGIC:='1';
           Amp : in integer range 0 to 4095:=1219;
           Step: in integer range 0 to 4095:=20; --Modulation Frequency at 60kHz is 19.6875 with high clock--Modulation Frequency at 60kHz
           Phase:in Integer range 0 to 16383:=0; --This is 16-bit
           Ch1,Ch2 : out  STD_LOGIC_VECTOR (11 downto 0);
           Digital_Ch1,Digital_Ch2 : out std_logic
           );
end Signal_Generator;

architecture Behavioral of Signal_Generator is

component Sine_RAM
   port (
   clka: IN std_logic;
   addra: IN std_logic_VECTOR(11 downto 0);
   douta: OUT std_logic_VECTOR(10 downto 0);
   clkb: IN std_logic;
   addrb: IN std_logic_VECTOR(11 downto 0);
   doutb: OUT std_logic_VECTOR(10 downto 0));
end component;


Signal Phase_1, Phase_2 :   std_logic_vector(11 downto 0);--sign converted quadrature phase
Signal Sine_A, Sine_B   :   std_logic_vector(10 downto 0);
Signal Phase_Counter_1, Phase_Counter_2   :   std_logic_vector(13 downto 0);--Raw accumulated phase
Signal Sine_1:   std_logic_vector(14+10 downto 0);


begin

Sine_LUT : Sine_RAM
      port map (
         clka => AWG_CLK,
         addra => Phase_1,
         douta => Sine_A,
         clkb => AWG_CLK,
         addrb => Phase_2,
         doutb => Sine_B
         );
with Phase_Counter_1(12) select
   Phase_1   <=   Phase_Counter_1(11 downto 0)                   when '0',
               "111111111111"-Phase_Counter_1(11 downto 0)    when Others;
with Phase_Counter_2(12) select
   Phase_2   <=   Phase_Counter_2(11 downto 0)                   when '0',
               "111111111111"-Phase_Counter_2(11 downto 0)    when Others;      
Sine_1(12+10 downto 0)<=Sine_A(10 downto 0)*conv_std_logic_vector(Amp,12);
--Sine_2<=Sine_B;

Digital_Ch1<=Phase_Counter_1(13);
Digital_Ch2<=Phase_Counter_2(13);

Read_Out_MUX:Process (AWG_CLK)    --Reset is built-in. no external reset is required.,SG_RST);
variable Sine_full_1,Sine_full_2,Square_Mod : std_logic_vector(11 downto 0);
Begin
--the Multiplexer for the phase
   if rising_edge(AWG_CLK)   then
      Case Phase_Counter_1(13) is
         When '0'   =>
            Sine_full_1   :=   '0'& Sine_1(12+10 downto 12); --the extra zero here is because we are covering only HALF
                                                   --of the range, so the overall amplitude need to be 11-bits
                                                   --instead of 12.
            Square_Mod := '0' & Conv_std_logic_vector(Amp/2,11);
         when Others   =>
            Sine_full_1   :=   "000000000000"- Sine_1(12+10 downto 12);
            Square_Mod := "000000000000"- '0' & Conv_std_logic_vector(Amp/2,11);
      end case;
      Case Phase_Counter_2(13) is
         When '0'   =>
            Sine_full_2   :=   '0' & Sine_B(10 downto 0);
         when Others   =>
            Sine_full_2   :=   "000000000000"-Sine_B(10 downto 0);
      end case;
      
      Case Square_Wave_Handle is
         when '0' =>
            Ch1<=Sine_full_1; 
            Ch2<=Sine_full_2; 
         When Others   =>
            Ch1<= Square_Mod;
            Ch2<=   Not(Phase_Counter_2(13)) & Phase_Counter_2(13) & Phase_Counter_2(13)
                     & Phase_Counter_2(13) & Phase_Counter_2(13) & Phase_Counter_2(13) 
                     & Phase_Counter_2(13) & Phase_Counter_2(13) & Phase_Counter_2(13)
                     & Phase_Counter_2(13) & Phase_Counter_2(13) & Phase_Counter_2(13) ; 
      end case;
   end if;
end process Read_Out_MUX;


Phase_Incrementor: Process(AWG_CLK)
variable Step0 : integer range 0 to 4095;
Begin      
      If falling_edge(AWG_CLK) THEN
         if Step=Step0 then
            Phase_Counter_1 <= Phase_Counter_1 + conv_std_logic_vector(Step,12); --conv_integer(SW(3 downto 0));
         else Step0:=Step;
            Phase_Counter_1 <= "00000000000000";
         end if;
         Phase_Counter_2 <= Phase_Counter_1 + conv_std_logic_vector(Phase,14);         
      end if;         
end process Phase_Incrementor;

end Behavioral;

